package eel6788.gui;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.geom.Point2D;

import yaes.framework.world.map.Location;
import yaes.framework.world.sensornetwork.AbstractSensorAgent;
import yaes.framework.world.sensornetwork.SensorNetworkWorld;
import yaes.framework.world.sensornetwork.SensorNode;
import yaes.ui.visualization.IPainter;
import yaes.ui.visualization.PainterHelper;
import yaes.ui.visualization.VisualMapPanel;
import yaes.ui.visualization.application.sensornetwork.PainterConfiguration;

/**
 * Implements the painter for a sensor node
 * @author lboloni
 *
 */
public class DirectedDiffusionNodePainter implements IPainter {
    protected PainterConfiguration painterConf;
	private final SensorNetworkWorld sensingManager;
	public int sensorNodeSize = 4;

	public DirectedDiffusionNodePainter(SensorNetworkWorld sensingManager,
			PainterConfiguration painterConf) {
		this.sensingManager = sensingManager;
		this.painterConf = painterConf;
	}

	
	public void paint(Graphics2D g, Object o, VisualMapPanel panel) {
		//if (!(o instanceof SensorNode)) {
		//	return;
		//}
		final SensorNode node = (SensorNode) o;
		final AbstractSensorAgent agent = node.getAgent();
		// Paint the node
		final Point2D.Double loc = node.getLocation().getRoundedLocation().asPoint();
		panel.getTheTransform().transform(loc, loc);
		final int x = (int) loc.getX();
		final int y = (int) loc.getY();
		PainterHelper.paintLabel(g, agent.getName(), node.getLocation().getRoundedLocation().asPoint(), 5, 5, panel);
		g.fill3DRect(x - sensorNodeSize / 2, y - sensorNodeSize / 2,
				sensorNodeSize, sensorNodeSize, true);
		// paint the sensing range
		if ((Boolean)painterConf.get(PainterConfiguration.PCONF_PAINT_SENSOR_RANGE)) {
			g.setColor((Color)painterConf.get(PainterConfiguration.PCONF_SENSOR_RANGE_COLOR));
			final Composite save = g.getComposite();
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
					(float) 0.2));
			int sensorRangeSize = (int)(panel.getTheTransform().getScaleX() * agent.getSensorRange()) ;			
			g.fillOval(x - sensorRangeSize, y - sensorRangeSize, 2 * sensorRangeSize,
					2 * sensorRangeSize);
			g.setComposite(save);
		}
		// paint the transmission range
		if ((Boolean)painterConf.get(PainterConfiguration.PCONF_PAINT_TRANSMISSION_RANGE)) {
			int transmissionRangeSize = (int)(panel.getTheTransform().getScaleX() * agent.getTransmissionRange()) ;			
			g.setColor((Color)painterConf.get(PainterConfiguration.PCONF_TRANSMISSION_RANGE_COLOR));
			g.drawOval(x - transmissionRangeSize,
					y - transmissionRangeSize, 2 * transmissionRangeSize, 2 * transmissionRangeSize);
		}
	}

	public int getLayer() {
		return 0;
	}
}
